<?php
/**
 * Mirasvit
 *
 * This source file is subject to the Mirasvit Software License, which is available at https://mirasvit.com/license/.
 * Do not edit or add to this file if you wish to upgrade the to newer versions in the future.
 * If you wish to customize this module for your needs.
 * Please refer to http://www.magentocommerce.com for more information.
 *
 * @category  Mirasvit
 * @package   mirasvit/module-email
 * @version   2.5.3
 * @copyright Copyright (C) 2023 Mirasvit (https://mirasvit.com/)
 */


declare(strict_types=1);

namespace Mirasvit\EmailDesigner\Api\Data;

interface TemplateInterface
{
    const TABLE_NAME = 'mst_email_designer_template';

    const ID                        = 'template_id';
    const TITLE                     = 'title';
    const SYSTEM_ID                 = 'system_id';
    const DESCRIPTION               = 'description';
    const TEMPLATE_SUBJECT          = 'template_subject';
    const TEMPLATE_AREAS            = 'template_areas';
    const TEMPLATE_AREAS_SERIALIZED = 'template_areas_serialized';
    const CREATED_AT                = 'created_at';
    const UPDATED_AT                = 'updated_at';

    /**
     * @return int
     */
    public function getId();

    public function getTitle(): string;

    public function setTitle(string $title): self;

    public function getDescription(): string;

    public function setDescription(string $description): self;

    public function getTemplateSubject(): string;

    public function getTemplateText(): string;

    public function setTemplateSubject(string $subject): self;

    public function getTheme(): ?ThemeInterface;

    public function setTheme(ThemeInterface $theme): self;

    public function getThemeId(): ?int;

    public function setThemeId(int $themeId): self;

    public function getAreas(): array;

    public function getTemplateAreas(): array;

    public function setTemplateAreas(array $data): self;

    public function getTemplateAreasSerialized(): string;

    public function setTemplateAreasSerialized(string $dataSerialized): self;

    public function getAreaText(string $code): ?string;

    public function setAreaText(string $code, string $content): self;

    public function getAreaCodeByContent(string $content): ?string;

    public function getCreatedAt(): string;

    public function setCreatedAt(string $createdAt): self;

    public function getUpdatedAt(): string;

    public function setUpdatedAt(string $updatedAt): self;

    public function isSystem(): bool;

    public function setSystemId(int $id): self;

    public function import(string $filePath): self;

    /**
     * Object data getter
     *
     * If $key is not defined will return all the data as an array.
     * Otherwise it will return value of the element specified by $key.
     * It is possible to use keys like a/b/c for access nested array data
     *
     * If $index is specified it will assume that attribute data is an array
     * and retrieve corresponding member. If data is the string - it will be explode
     * by new line character and converted to array.
     *
     * @param string     $key
     * @param string|int $index
     * @return mixed
     */
    public function getData($key = '', $index = null);

    /**
     * If $key is empty, checks whether there's any data in the object
     * Otherwise checks if the specified attribute is set.
     *
     * @param string $key
     * @return bool
     */
    public function hasData($key = '');

    /**
     * Overwrite data in the object.
     *
     * The $key parameter can be string or array.
     * If $key is string, the attribute value will be overwritten by $value
     *
     * If $key is an array, it will overwrite all the data in the object.
     *
     * @param string|array  $key
     * @param mixed         $value
     *
     * @return $this
     */
    public function setData($key, $value = null);

    /**
     * Add data to the object.
     *
     * Retains previous data in the object.
     *
     * @param array $arr
     *
     * @return $this
     */
    public function addData(array $arr);
}
